module Core.Types where

import qualified Data.Text                      as T
import qualified Data.Text.Lazy                 as LT

import           Control.Monad.Reader           (MonadIO, MonadReader, ReaderT)
import           Data.Text
import           Database.SQLite.Simple         (ToRow)
import           Database.SQLite.Simple.FromRow (FromRow)
import           GHC.Generics                   (Generic)
import           Numeric.Natural                (Natural)
import           Text.Blaze.Html
import           Web.Scotty.Trans               (ActionT, ScottyT)

type PurrApp    a = ScottyT LT.Text ConfigM a
type PurrAction a = ActionT LT.Text ConfigM a
type Random     a = IO a

newtype ConfigM a = ConfigM
  { runConfigM :: ReaderT DhallConfig IO a
  } deriving (Applicative, Functor, Monad, MonadIO, MonadReader DhallConfig)

newtype Password = Password' String
  deriving (Eq, Ord)

newtype UserSecret = UserSecret' String
  deriving (Eq, Ord)

newtype GenLink = GenLink' String
  deriving (Eq, Ord)

data DhallConfig = DhallConfig
  { environment     :: String
  , applicationHost :: String
  , applicationPort :: Int
  , dbFile          :: String
  , dbKey           :: String
  , linkLength      :: Int
  , adminEmail      :: String
  } deriving (Generic, Show)

data SecretEntry = SecretEntry
  { link     :: T.Text
  , secret   :: T.Text
  , date     :: Integer
  , life     :: Integer
  , views    :: Integer
  , maxViews :: Integer
  } deriving (Show, Generic)

data Secret = Password | UserSecret
    deriving (Show, Generic)

instance Show GenLink where
  show (GenLink' a) = a

instance Show Password where
  show (Password' a) = a

instance Show UserSecret where
  show (UserSecret' a) = a

instance ToMarkup Password where
  toMarkup = toMarkup . show

instance ToMarkup UserSecret where
  toMarkup = toMarkup . show

instance ToMarkup Secret where
  toMarkup = toMarkup . show

instance FromRow  SecretEntry where
instance ToRow    SecretEntry where
