{-# LANGUAGE TemplateHaskell #-}

module Feature.Generation.Passwords
  ( Password
  , Random
  , suggestedScheme
  , xkcd
  , oldschool
  , gibberish
  ) where

import           Core.Types
import           Feature.Generation.Shared (camelCase, rCharSym, rIndex,
                                            validChars, validNumbers,
                                            validSymbols)

import           Data.FileEmbed
import           Data.List                 (singleton)

suggestedScheme :: Int -> Random Password
suggestedScheme i
  | i > 17    = xkcd
  | i > 12    = oldschool
  | otherwise = gibberish i

xkcd :: Random Password
xkcd = do
  wOne   <- rCamel
  wTwo   <- rCamel
  wThree <- rCamel
  wFour  <- rCamel
  return  $ Password' (wOne <> wTwo <> wThree <> wFour)

oldschool :: Random Password
oldschool = do
  wOne <- rCamel
  wTwo <- rCamel
  nOne <- rNum
  nTwo <- rNum
  nThr <- rNum
  nFou <- rNum
  sOne <- rSym
  return
    $ Password' (wOne <> wTwo
   <> show nOne <> show nTwo <> show nThr <> show nFou
   <> pure sOne)

gibberish :: Int -> Random Password
gibberish i = go i (return "")
  where
    go :: Int -> Random String -> Random Password
    go 0 s = Password' <$> s
    go i s = go (i - 1) (s <> (singleton <$> rCharSym))

rNum :: Random Int
rNum = rIndex validNumbers

rWord :: Random String
rWord = rIndex wordList

rSym :: Random Char
rSym = rIndex validSymbols

rCamel :: Random String
rCamel = camelCase <$> rWord

wordList :: [String]
wordList = lines $(embedStringFile "./assets/wordlist.txt")
