module Feature.Generation.Shared where

import           Data.Char     (intToDigit, toLower, toUpper)
import           System.Random (randomRIO)

rIndex :: [a] -> IO a
rIndex arr = randomRIO (0, length arr - 1)
  >>= (\i -> return $ arr !! i)

rCap :: Char -> IO Char
rCap c = rIndex [True,False] >>= (\r -> return $ rCap' r c)
  where
    rCap' :: Bool -> Char -> Char
    rCap' True  c = toUpper c
    rCap' False c = toLower c

rChar :: IO Char
rChar = rIndex validChars >>= rCap

rCharSym :: IO Char
rCharSym = rIndex (validChars <> validSymbols)>>= rCap

camelCase :: [Char] -> [Char]
camelCase [] = []
camelCase x  = toUpper (head x) : map toLower (tail x)

validChars :: [Char]
validChars = validLetters <> fmap intToDigit validNumbers

validNumbers :: [Int]
validNumbers = [1..9]

validLetters :: [Char]
validLetters = ['a'..'z']

validSymbols :: [Char]
validSymbols = ['!', '@', '#', '$', '%', '^', '&', '*']
