module Feature.Sharing.HTTP ( routes ) where

import           Core.SQLite               (confLinkLength)
import           Core.Templates            (renderIndex)
import           Core.Types

import           Feature.Generation.Links  (genLink)
import           Feature.Sharing.SQLite    (findByLink, insertNewSecret)
import           Feature.Sharing.Templates (renderPw)

import           Control.Monad.Reader      (ask, lift, liftIO)
import           Data.List.Split           (splitOn)
import           Data.Maybe                (listToMaybe)
import           Prelude
import           Web.Scotty.Trans

import qualified Data.Text                 as T
import qualified Data.Text.Lazy            as LT


routes :: PurrApp ()
routes = do

  get "/pw/:id" $ do
    reqId <- param "id"
    conf  <- lift ask
    html   $ renderIndex reqId (adminEmail conf)

  post "/pw" $ do
    reqId <- param "userLink"
    res   <- findByLink reqId
    html   $ renderPw (last $ splitOn "/" reqId) res

  post "/new" $ do
    reqSecret <- param "newSec"
    reqDur    <- param "newSecDuration"
    reqViews  <- param "newSecViews"
    cLength   <- confLinkLength
    link      <- liftIO $ genLink cLength
    insertNewSecret reqSecret reqDur (T.pack $ show link) reqViews
    html       $ renderPw (show link) (Just reqSecret)
